/*
 * This file is part of qtutil
 * Copyright (c) 2000 David Deaven (deaven@execpc.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Author: deaven $
 * $Id: QTFrameExtractor.cxx,v 1.2 2000/10/14 04:47:06 deaven Exp $
 */


#include "QTFrameExtractor.h"
#include "QTAtomImpl.h"

int QTFrameExtractor::extract(int frameno, FILE* fout)
{
    /* Find the "stbl" atoms containing JPEG data.
     */

    int frame = 0;

    int num_stbl, i_stbl;
    QTAtom* stbl = Root.findChildrenOfType("stbl", &num_stbl);
    for (i_stbl = 0; i_stbl < num_stbl; i_stbl++)
    {
	int num_stsd, i_stsd;
	QTAtom* stsd = stbl[i_stbl].findChildrenOfType("stsd", &num_stsd);
	for (i_stsd = 0; i_stsd < num_stsd; i_stsd++)
	{
	    if (QTAtomImpl::cmp32(((QTstsd*) stsd[i_stsd].impl())->samples[0].format, "jpeg"))
	    {
		QTAtom stsc = stbl[i_stbl].findChildOfType("stsc");
		QTAtom stsz = stbl[i_stbl].findChildOfType("stsz");
		QTAtom stco = stbl[i_stbl].findChildOfType("stco");

		int32* sample_size = 
		    ((QTstsz*) stsz.impl())->sizes;
		int32  num_samples = 
		    ((QTstsz*) stsz.impl())->count;

		int32 max_sample_size = 0;
		int32 sample;
		for (sample = 0; sample < num_samples; sample++)
		{
		    if (sample_size[sample] > max_sample_size)
		    {
			max_sample_size = sample_size[sample];
		    }
		}

		byte* sample_data = new byte[max_sample_size];

		// loop through each sample
		int32 num_chunks = 
		    ((QTstco*) stco.impl())->count;
		int32 chunk;

		int32 sample_to_chunk_index = 0;
		sample = 0;
		for (chunk = 0; chunk < num_chunks; chunk++)
		{
		    // next chunk in stsc table?
		    if ((sample_to_chunk_index < ((QTstsc*) stsc.impl())->count) &&
		        (chunk + 1) == ((QTstsc*) stsc.impl())->sample_to_chunk[sample_to_chunk_index + 1].first_chunk)
		    {
			sample_to_chunk_index++;
		    }

		    int32 samples_per_chunk = 
			((QTstsc*) stsc.impl())->sample_to_chunk[sample_to_chunk_index].samples_per_chunk;

		    int offset = 
			((QTstco*) stco.impl())->chunk_offsets[chunk];
		    fseek(stbl[i_stbl].impl()->filePtr(), offset, SEEK_SET);
		    int32 lsample;
		    for (lsample = 0; lsample < samples_per_chunk; lsample++)
		    {
			fread(sample_data, sample_size[sample], 1, stbl[i_stbl].impl()->filePtr());
			frame++;
			if (frameno == frame)
			{
			    fwrite(sample_data, sample_size[sample], 1, fout);
			}
			sample++;
		    }
		}
	    }
	}
    }

    return 0;
}

int QTFrameExtractor::createTime()
{
    QTAtom mvhd = Root.findChildOfType("mvhd");
    QTmvhd* impl = (QTmvhd*) mvhd.impl();
    if (impl)
    {
	time_t t = impl->convert_mac_time(impl->create_time);
	return (int) t;
    }

    return 0;
}

int QTFrameExtractor::size()
{
    /* Find the "stbl" atoms containing JPEG data.
     */

    int frames = 0;

    int num_stbl, i_stbl;
    QTAtom* stbl = Root.findChildrenOfType("stbl", &num_stbl);
    for (i_stbl = 0; i_stbl < num_stbl; i_stbl++)
    {
	int num_stsd, i_stsd;
	QTAtom* stsd = stbl[i_stbl].findChildrenOfType("stsd", &num_stsd);
	for (i_stsd = 0; i_stsd < num_stsd; i_stsd++)
	{
	    if (QTAtomImpl::cmp32(((QTstsd*) stsd[i_stsd].impl())->samples[0].format, "jpeg"))
	    {
		QTAtom stsz = stbl[i_stbl].findChildOfType("stsz");

		int32  num_samples = 
		    ((QTstsz*) stsz.impl())->count;

		frames += num_samples;
	    }
	}
    }

    return frames;
}

int QTFrameExtractor::extractAll(const char* prefix)
{
    /* Find the "stbl" atoms containing JPEG data.
     */

    int frame = 0;

    int num_stbl, i_stbl;
    QTAtom* stbl = Root.findChildrenOfType("stbl", &num_stbl);
    for (i_stbl = 0; i_stbl < num_stbl; i_stbl++)
    {
	int num_stsd, i_stsd;
	QTAtom* stsd = stbl[i_stbl].findChildrenOfType("stsd", &num_stsd);
	for (i_stsd = 0; i_stsd < num_stsd; i_stsd++)
	{
	    if (QTAtomImpl::cmp32(((QTstsd*) stsd[i_stsd].impl())->samples[0].format, "jpeg"))
	    {
		QTAtom stsc = stbl[i_stbl].findChildOfType("stsc");
		QTAtom stsz = stbl[i_stbl].findChildOfType("stsz");
		QTAtom stco = stbl[i_stbl].findChildOfType("stco");

		int32* sample_size = 
		    ((QTstsz*) stsz.impl())->sizes;
		int32  num_samples = 
		    ((QTstsz*) stsz.impl())->count;

		int32 max_sample_size = 0;
		int32 sample;
		for (sample = 0; sample < num_samples; sample++)
		{
		    if (sample_size[sample] > max_sample_size)
		    {
			max_sample_size = sample_size[sample];
		    }
		}

		byte* sample_data = new byte[max_sample_size];

		// loop through each sample
		int32 num_chunks = 
		    ((QTstco*) stco.impl())->count;
		int32 chunk;

		int32 sample_to_chunk_index = 0;
		sample = 0;
		for (chunk = 0; chunk < num_chunks; chunk++)
		{
		    // next chunk in stsc table?
		    if ((sample_to_chunk_index < ((QTstsc*) stsc.impl())->count) &&
		        (chunk + 1) == ((QTstsc*) stsc.impl())->sample_to_chunk[sample_to_chunk_index + 1].first_chunk)
		    {
			sample_to_chunk_index++;
		    }

		    int32 samples_per_chunk = 
			((QTstsc*) stsc.impl())->sample_to_chunk[sample_to_chunk_index].samples_per_chunk;

		    printf("chunk %d of %d (%d samples)...\n", chunk + 1, num_chunks, samples_per_chunk);
		    fflush(stdout);
		    int offset = 
			((QTstco*) stco.impl())->chunk_offsets[chunk];
		    fseek(stbl[i_stbl].impl()->filePtr(), offset, SEEK_SET);
		    int32 lsample;
		    for (lsample = 0; lsample < samples_per_chunk; lsample++)
		    {
			fread(sample_data, sample_size[sample], 1, stbl[i_stbl].impl()->filePtr());
			char buf[256];
			sprintf(buf, "%s%06d.jpg", prefix, frame++);
			FILE* fout = fopen(buf, "w");
			fwrite(sample_data, sample_size[sample], 1, fout);
			fclose(fout);
			sample++;
		    }
		}
	    }
	}
    }

    return 0;
}

