/*
 * This file is part of qtutil
 * Copyright (c) 2000 David Deaven (deaven@execpc.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Author: deaven $
 * $Id: qtdump.cxx,v 1.5 2000/10/14 04:47:06 deaven Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "QTAtom.h"
#include "QTFrameExtractor.h"

/* $Id: qtdump.cxx,v 1.5 2000/10/14 04:47:06 deaven Exp $
 */

static void usage()
{
    fprintf(stderr,
	"Usage: qtdump <QuickTime-file> [options]\n"
	"options:\n"
	"    -size           write the number of JPEG frames to stdout\n"
	"    -frame N        write JPEG data for frame N to stdout\n"
	"    -extract PFX    write all JPEG frames to file <PFX>NNNNNN.jpg\n"
	"    -tree           print the QuickTime file tree to stdout\n"
	"    -time           print the Unix time value for when this file was created\n"
	);
}

int main(int argc, char** argv)
{
    QTAtom root;
    int arg;

    for (arg = 1; arg < argc; arg++)
    {
	if (!strcmp("-tree", argv[arg]))
	{
	    printf("File contents:\n");
	    root.print();
	}
	else if (!strcmp("-extract", argv[arg]))
	{
	    QTFrameExtractor e(root);
	    e.extractAll(argv[++arg]);
	}
	else if (!strcmp("-size", argv[arg]))
	{
	    QTFrameExtractor e(root);
	    printf("%d\n", e.size());
	}
	else if (!strcmp("-frame", argv[arg]))
	{
	    /* Skip over leading non-numerics, to allow for 
	     * use with mpeg_encode, which might give us a 
	     * frame number disguised as a relative file path
	     * such as "./23" for frame 23.
	     */
	    char* s = argv[++arg];
	    while (isprint(*s) && !isdigit(*s)) s++;
	    int frameno = atoi(s);
	    QTFrameExtractor e(root);
	    if (frameno < 1 || frameno > e.size())
	    {
		fprintf(stderr, "Bad frame number %d (must be in range 1-%d)\n",
		    frameno, e.size());
		return 1;
	    }
	    e.extract(frameno, stdout);
	}
	else if (!strcmp("-time", argv[arg]))
	{
	    QTFrameExtractor e(root);
	    printf("%d\n", e.createTime());
	}
	else if (!strcmp("-h", argv[arg]))
	{
	    usage();
	    return 0;
	}
	else if (argv[arg][0] == '-')
	{
	    fprintf(stderr, "unknown option \"%s\"\n", argv[arg]);
	    usage();
	    return 1;
	}
	else
	{
	    FILE* fin = fopen(argv[arg], "r");
	    if (!fin)
	    {
		fprintf(stderr, "Cannot open \"%s\"\n", argv[arg]);
		return 1;
	    }
	    root = QTAtom::parseFile(fin);
	}
    }

    return 0;
}

